%--------------------------------------------------------------------------
%   Very basic life-cycle (OLG) model 
%   - Deterministic life-cycle income (i.e. no uncertainty)
%   - Savings subject to a borrowing limit 
%   - Solution: 
%       - Savings are chosen/forced onto a pre-determined gridspace 
%       - Works well for very simple models 
%       - Does not work so well for more complicated models 
%   - Simulate a panel of households to approximate the distribution of
%   household assets by age. 
%
%--------------------------------------------------------------------------
%% Setup

modelname = 'mod1';
figdir    = './../Figures/';


%% Parameters 
beta  = 0.95;        % Discount factor 
r     = 1/beta-1;    % Interest rate 
ages = [1,2,3];      % Ages 
J    = length(ages); % Number of ages 

y_j = [0.95, 1.10, 0.95];  % Life-cycle income path


%% Setup asset gridspace 

Na    = 25;   % Number of asset grid points 
amin  = -0.9; % Minimum value of assets on the grid 
amax  = 5;    % Maximum value of assets on the grid 

agrid = linspace(amin, amax, Na)';   % Simple linearly spaced asset grid 

figure; 
plot(1:Na, agrid, 'linewidth', 2, 'color', 'blue', 'Marker', 'o'); hold on;
plot([1, Na], [0,0], 'color', 'black');
yy = ylim;
ylim([yy(1)-0.1*abs(yy(1)), yy(2)+0.1*abs(yy(2))])
xlabel('Grid number')
ylabel('Asset value at grid')
title('Asset Grid Space')
set(gca,'box','off')
legend(['$N_a=$', num2str(Na)])

f_height_div  = 1.25;
func_latexfig(f_height_div)


filename = ['Figure_', modelname, '_assetgridspace.pdf'];
exportgraphics(gcf, [figdir, filename]);




%% 0. Pre-allocate value function and choice function vectors

V_j      = zeros(Na, J);
aprime_j = zeros(Na, J);
cons_j   = zeros(Na, J);


%-----------------------------
%% 1. Solve final period of life first: j = J
%-----------------------------

jj = J;  % Set final age 

aprime_j(:,jj) = zeros(Na, 1);              % No saving in last period of life 
cons_j(:,jj)   = y_j(jj) + agrid.*(1+r);    % Consume everything in last period of life 
V_j(:,jj)      = log( cons_j(:,jj) );       % Utility over consumption 

% Often want to ensure that V_j is never -inf or nan. 
scale     = -1e+10;    % Very large negative number 
V_j(:,jj) = (cons_j(:,jj)>0).*V_j(:,jj) + (cons_j(:,jj)<=0).*scale;


%-----------------------------
%% 2. Solve for all previous ages: j = J-1, ..., 1
%-----------------------------
%   1. Must choose aprime from the grid of possible options: agrid
%      Since V_j(jj+1) is defined at all agrid values, simply find value
%      corresponding to chosen aprime value 
%   2. Basic solution method: grid search. Note, this is VERY slow for large models!

% Auxilliary functions 
util_func = @(cons) log( cons );

% Loop *backwards* through ages 
for jj = J-1:-1:1   
    % Loop across each asset grid
    for aa = 1:Na
        aprime     = agrid;                                % Suppose savings choice comes from the asset grid
        cons       = y_j(jj) + agrid(aa).*(1+r) - aprime;  % Get consumption from the budget constraint
        cons       = (cons>0).*cons + (cons<=0).*1e-10;    % Trick to make sure consumption always positive

        V_jplus1   = V_j( :, jj+1); % Get tomorrow's value function where aprime=current asset grid
        V_on_agrid = util_func(cons) + beta.*V_jplus1;   % Compute today's value function 
        
        % Find index on asset grid corresponding to maximum value function value
        [~, ix_max] = max(V_on_agrid);        
        
        % Get optimal functions using position on asset grid with maximum value 
        aprime_j(aa,jj) = aprime(ix_max);
        cons_j(aa,jj)   = cons(ix_max);
        V_j(aa,jj)      = V_on_agrid(ix_max);        
    end
end


%% Plot value and choice functions 



figure;

subplot(3,1,1)
plot(agrid, V_j(:,1), 'linewidth', 2, 'color', 'blue'); hold on;
plot(agrid, V_j(:,2), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot(agrid, V_j(:,3), 'linewidth', 2, 'color', [0, 0.5, 0], 'LineStyle', ':');
yy = ylim;
ylim([yy(1)-0.1*abs(yy(1)), yy(2)+0.1*abs(yy(2))])
xlim([agrid(1), agrid(end)])
legend('Age $j=1$', 'Age $j=2$', 'Age $j=3$', 'location', 'southeast');
legend boxoff 
set(gca,'box','off')
title('Value Function')
xlh = xlabel('Asset grid');
xlh.Position(1) = xlh.Position(1) + 4.75;

subplot(3,1,2)
plot(agrid, cons_j(:,1), 'linewidth', 2, 'color', 'blue'); hold on;
plot(agrid, cons_j(:,2), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot(agrid, cons_j(:,3), 'linewidth', 2, 'color', [0, 0.5, 0], 'LineStyle', ':');
yy = ylim;
ylim([yy(1)-0.1*abs(yy(1)), yy(2)+0.1*abs(yy(2))])
xlim([agrid(1), agrid(end)])
set(gca,'box','off')
xlh = xlabel('Asset grid');
xlh.Position(1) = xlh.Position(1) + 4.75;
title('Consumption Function')

subplot(3,1,3)
plot(agrid, aprime_j(:,1), 'linewidth', 2, 'color', 'blue'); hold on;
plot(agrid, aprime_j(:,2), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot(agrid, aprime_j(:,3), 'linewidth', 2, 'color', [0, 0.5, 0], 'LineStyle', ':');
yy = ylim;
ylim([yy(1)-0.1*abs(yy(1)), yy(2)+0.1*abs(yy(2))])
xlim([agrid(1), agrid(end)])
set(gca,'box','off')
xlh = xlabel('Asset grid');
xlh.Position(1) = xlh.Position(1) + 4.75;
title('Asset Function')




f_height_div  = 1.25;
func_latexfig(f_height_div)

filename = ['Figure_', modelname, '_policyfunc.pdf'];
exportgraphics(gcf, [figdir, filename]);




%-----------------------------
%% 3. Simulate households 
%-----------------------------
% Now, simulate the distribution of households across all age groups. 
% Start with an initial distribution over assets (could be made up, or drawn from data) 
% Take a fixed size of population, and use the decision rules at each age
% to compute what their states will be in the next period. Simulate from
% j=1,...,J.
 
Npop = 10000; % Need a large number of households 

% Initialize sim vectors
sim_assets = zeros(Npop, J);
sim_cons   = zeros(Npop, J);

% Draw sample of initial assets 
mu_a  = 0;    % Mean of asset dist in period 1
sig_a = 0.2;    % std dev of asset dist in period 1
sim_assets(:,1) = mu_a + sig_a.*randn(Npop,1);

% Force assets onto grid 
[~,ix_max] = min(abs(agrid-sim_assets(:,1)'));
sim_assets(:,1) = agrid(ix_max);

% Simulate panel one age group at a time
for jj = 1:J
    % Loop over each household
    for nn=1:Npop
        % Get position in asset grid
        ix_max = (sim_assets(nn,jj)==agrid);

        % Find choice of assets/consumption for each household at each grid point
        sim_assets(nn,jj+1) = aprime_j(ix_max,jj);
        sim_cons(nn,jj)     = cons_j(ix_max,jj);
    end    
end
    


%-------------------------
%% Plot life-cycle simulation
%-------------------------

figure;

subplot(3,1,1)
plot(ages, y_j, 'linewidth', 2, 'color', 'blue', 'Marker', 'o'); hold on;
yy = ylim;
ylim([yy(1)*0.9, yy(2)*1.1])
xticks(ages)
title('Life-cycle income')
set(gca,'box','off')

subplot(3,1,2)
plot(ages, mean(sim_cons), 'linewidth', 2, 'color', 'blue', 'Marker', 'o'); hold on;
plot(ages, mean(sim_cons)-1*std(sim_cons), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot(ages, mean(sim_cons)+1*std(sim_cons), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
yy = ylim;
ylim([yy(1)*0.9, yy(2)*1.1])
xticks(ages)
set(gca,'box','off')
title('Life-cycle consumption')


subplot(3,1,3)
plot(ages, mean(sim_assets(:,2:J+1)), 'linewidth', 2, 'color', 'blue', 'Marker', 'o'); hold on;
plot(ages, mean(sim_assets(:,2:J+1))-1*std(sim_assets(:,2:J+1)), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot(ages, mean(sim_assets(:,2:J+1))+1*std(sim_assets(:,2:J+1)), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
yy = ylim;
ylim([yy(1)-0.1*abs(yy(1)), yy(2)+0.1*abs(yy(2))])
xticks(ages)
title('Life-cycle assets')
legend('Mean', 'Mean -/+ 1s.d.', 'location', 'southeast');
legend boxoff 
set(gca,'box','off')



f_height_div  = 1.25;
func_latexfig(f_height_div)

filename = ['Figure_', modelname, '_simlifecycle.pdf'];
exportgraphics(gcf, [figdir, filename]);


%-------------------------
%% Plot simulated distributions
%-------------------------

figure;

subplot(3,1,1)
hist(sim_assets(:,1), 100);
title('Distribution of Initial Assets')
set(gca,'box','off')

subplot(3,1,2)
hist(sim_cons(:), 100);
set(gca,'box','off')
title('Distribution of consumption')

subplot(3,1,3)
hist(sim_assets(:), 100);
title('Distribution of assets')
set(gca,'box','off')

f_height_div  = 1.25;
func_latexfig(f_height_div)

filename = ['Figure_', modelname, '_simdist.pdf'];
exportgraphics(gcf, [figdir, filename]);



