%--------------------------------------------------------------------------
%   Very basic life-cycle (OLG) model 
%   - Deterministic life-cycle income (i.e. no uncertainty)
%   - Savings subject to a borrowing limit 
%   - Solution: 
%       - Savings are chosen/forced onto a pre-determined gridspace 
%       - Works well for very simple models 
%       - Does not work so well for more complicated models 
%   - Simulate a panel of households to approximate the distribution of
%   household assets by age. 
%
%--------------------------------------------------------------------------
%% Setup

modelname = 'mod2';
figdir    = './../Figures/';

%% Parameters 
beta  = 0.95;    % Discount factor 
r     = 1/beta-1;    % Interest rate 
a_1   = 0;       % Initial assets 
ages = [1,2,3];  % Ages 
J    = length(ages); % Number of ages 

y_j = [0.95, 1.10, 0.95];  % Life-cycle incomes


%% Setup asset gridspace 

tic 

Na    = 1000; % Number of asset grid points 
amin  = -0.9;  % Minimum value of assets on the grid 
amax  = 10;  % Maximum value of assets on the grid 

agrid = linspace(amin, amax, Na)';   % Simple linearly space asset grid 

%% 0. Pre-allocate value function and choice function vectors

V_j      = zeros(Na, J);
aprime_j = zeros(Na, J);
cons_j   = zeros(Na, J);


%-----------------------------
%% 1. Solve final period of life first: j = J
%-----------------------------

jj = J;  % Set final age 

aprime_j(:,jj) = zeros(Na, 1);              % No saving in last period of life 
cons_j(:,jj)   = y_j(jj) + agrid.*(1+r);    % Consume everything in last period of life 
V_j(:,jj)      = log( cons_j(:,jj) );       % Utility over consumption 

% Often want to ensure that V_j is never -inf or nan. 
scale     = -1e+10;    % Very large negative number 
V_j(:,jj) = (cons_j(:,jj)>0).*V_j(:,jj) + (cons_j(:,jj)<=0).*scale;


%-----------------------------
%% 2. Solve for all previous ages: j = J-1, ..., 1
%-----------------------------
%   1. Use maximization algorithm to find optimal savings choice 
%       - We use "golden section search" (https://en.wikipedia.org/wiki/Golden-section_search)
%       - GSS is useful because it can be *vectorized*, which massively
%       speeds up the process 
%       - You could use other algorithms, however: Matlab's "fmincon", Newtown methods, hill-climbing methods,
%       Nelder-Meade, etc. Each have pros and cons, but can be difficult to
%       vectorize 
%   2. Now, since we only have values for V_j(jj+1) on the grid, we need to
%   "interpolate" the values of V_j(jj+1) between grid points (https://en.wikipedia.org/wiki/Linear_interpolation)

% Auxilliary functions 
util_func = @(cons) log( cons );

% Loop *backwards* through ages 
for jj = J-1:-1:1
    % Get bounds for asset choice
    lb = amin*ones(Na,1);          % Lower bound
    ub = (y_j(jj) + agrid.*(1+r)); % Upper bound

    % Value function given age, consumption, asset choice 
    valprime_func = @(jj, aprime) interp1(agrid, V_j(:,jj+1), aprime);
    val_func      = @(jj, cons, aprime) util_func(cons) + beta.*valprime_func(jj, aprime);    

    % Wrap value function for use in optimization routine below
    cons_func = @(astar) y_j(jj) + agrid.*(1+r) - astar;
    obj       = @(astar) val_func(jj, cons_func(astar), astar);

    % Use golden-section-search to find optimum asset choice
    [aprime_j(:,jj), V_j(:,jj)] = goldenx(obj,lb,ub);    
    
    % Given optimal choice of assets, reconstruct consumption from budget constraint
    cons_j(:,jj)   = cons_func(aprime_j(:,jj));
end


toc

%% Plot value and choice functions 



figure;

subplot(3,1,1)
plot(agrid, V_j(:,1), 'linewidth', 2, 'color', 'blue'); hold on;
plot(agrid, V_j(:,2), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot(agrid, V_j(:,3), 'linewidth', 2, 'color', [0, 0.5, 0], 'LineStyle', ':');
yy = ylim;
ylim([yy(1)-0.1*abs(yy(1)), yy(2)+0.1*abs(yy(2))])
xlim([agrid(1), agrid(end)])
legend('Age $j=1$', 'Age $j=2$', 'Age $j=3$', 'location', 'southeast');
legend boxoff 
set(gca,'box','off')
title('Value Function')
xlh = xlabel('Asset grid');
xlh.Position(1) = xlh.Position(1) + 4.75;

subplot(3,1,2)
plot(agrid, cons_j(:,1), 'linewidth', 2, 'color', 'blue'); hold on;
plot(agrid, cons_j(:,2), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot(agrid, cons_j(:,3), 'linewidth', 2, 'color', [0, 0.5, 0], 'LineStyle', ':');
yy = ylim;
ylim([yy(1)-0.1*abs(yy(1)), yy(2)+0.1*abs(yy(2))])
xlim([agrid(1), agrid(end)])
set(gca,'box','off')
xlh = xlabel('Asset grid');
xlh.Position(1) = xlh.Position(1) + 4.75;
title('Consumption Function')

subplot(3,1,3)
plot(agrid, aprime_j(:,1), 'linewidth', 2, 'color', 'blue'); hold on;
plot(agrid, aprime_j(:,2), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot(agrid, aprime_j(:,3), 'linewidth', 2, 'color', [0, 0.5, 0], 'LineStyle', ':');
yy = ylim;
ylim([yy(1)-0.1*abs(yy(1)), yy(2)+0.1*abs(yy(2))])
xlim([agrid(1), agrid(end)])
set(gca,'box','off')
xlh = xlabel('Asset grid');
xlh.Position(1) = xlh.Position(1) + 4.75;
title('Asset Function')




f_height_div  = 1.25;
func_latexfig(f_height_div)

filename = ['Figure_', modelname, '_policyfunc.pdf'];
exportgraphics(gcf, [figdir, filename]);





%-----------------------------
%% 3. Simulate households 
%-----------------------------
% Now, simulate the distribution of households across all age groups. 
% Start with an initial distribution over assets (could be made up, or drawn from data) 
% Take a fixed size of population, and use the decision rules at each age
% to compute what their states will be in the next period. Simulate from
% j=1,...,J.
 
Npop = 10000; % Need a large number of households 

% Initialize sim vectors
sim_assets = zeros(Npop, J);
sim_cons   = zeros(Npop, J);

% Draw sample of initial assets 
mu_a  = 0;    % Mean of asset dist in period 1
sig_a = 0.2;    % std dev of asset dist in period 1
sim_assets(:,1) = mu_a + sig_a.*randn(Npop,1);

% Simulate panel one age group at a time
for jj = 1:J
    % Use interpolation over asset function to find asset choices
    sim_assets(:,jj+1) = interp1(agrid, aprime_j(:,jj), sim_assets(:,jj));
    sim_cons(:,jj)     = y_j(jj) + sim_assets(:,jj).*(1+r) - sim_assets(:,jj+1);    
end
    

%-------------------------
%% Plot life-cycle simulation
%-------------------------

figure;

subplot(3,1,1)
plot(ages, y_j, 'linewidth', 2, 'color', 'blue', 'Marker', 'o'); hold on;
yy = ylim;
ylim([yy(1)*0.9, yy(2)*1.1])
xticks(ages)
title('Life-cycle income')
set(gca,'box','off')

subplot(3,1,2)
plot(ages, mean(sim_cons), 'linewidth', 2, 'color', 'blue', 'Marker', 'o'); hold on;
plot(ages, mean(sim_cons)-1*std(sim_cons), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot(ages, mean(sim_cons)+1*std(sim_cons), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
yy = ylim;
ylim([yy(1)*0.9, yy(2)*1.1])
xticks(ages)
set(gca,'box','off')
title('Life-cycle consumption')


subplot(3,1,3)
plot(ages, mean(sim_assets(:,2:J+1)), 'linewidth', 2, 'color', 'blue', 'Marker', 'o'); hold on;
plot(ages, mean(sim_assets(:,2:J+1))-1*std(sim_assets(:,2:J+1)), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot(ages, mean(sim_assets(:,2:J+1))+1*std(sim_assets(:,2:J+1)), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
yy = ylim;
ylim([yy(1)-0.1*abs(yy(1)), yy(2)+0.1*abs(yy(2))])
xticks(ages)
title('Life-cycle assets')
legend('Mean', 'Mean -/+ 1s.d.', 'location', 'southeast');
legend boxoff 
set(gca,'box','off')



f_height_div  = 1.25;
func_latexfig(f_height_div)

filename = ['Figure_', modelname, '_simlifecycle.pdf'];
exportgraphics(gcf, [figdir, filename]);





%-------------------------
%% Plot simulated distributions
%-------------------------

figure;

subplot(3,1,1)
hist(sim_assets(:,1), 100);
title('Distribution of Initial Assets')
set(gca,'box','off')

subplot(3,1,2)
hist(sim_cons(:), 100);
set(gca,'box','off')
title('Distribution of consumption')

subplot(3,1,3)
hist(sim_assets(:), 100);
title('Distribution of assets')
set(gca,'box','off')

f_height_div  = 1.25;
func_latexfig(f_height_div)

filename = ['Figure_', modelname, '_simdist.pdf'];
exportgraphics(gcf, [figdir, filename]);







