%--------------------------------------------------------------------------
%   Very basic life-cycle (OLG) model 
%   - Deterministic life-cycle income (i.e. no uncertainty)
%   - Savings subject to a borrowing limit 
%   - Solution: 
%       - Savings are chosen/forced onto a pre-determined gridspace 
%       - Works well for very simple models 
%       - Does not work so well for more complicated models 
%   - Simulate a panel of households to approximate the distribution of
%   household assets by age. 
%
%--------------------------------------------------------------------------
%% Setup

modelname = 'mod3';
figdir    = './../Figures/';


%% Parameters 
beta  = 0.95;    % Discount factor 
r     = 1/beta-1;    % Interest rate 
a_1   = 0;       % Initial assets 
ages = [1,2,3];  % Ages 
J    = length(ages); % Number of ages 

y_j = [0.95, 1.10, 0.95];  % Life-cycle incomes


%% Setup asset gridspace 

tic 

Na    = 1000; % Number of asset grid points 
amin  = -0.3;  % Minimum value of assets on the grid 
amax  = 10;  % Maximum value of assets on the grid 

agrid = linspace(amin, amax, Na)';   % Simple linearly space asset grid 


Nz    = 100; % Number of income grid points 
zmin  = 0.5;  % Minimum value of income on the grid 
zmax  = 5;  % Maximum value of income on the grid 
zgrid = linspace(zmin, zmax, Nz)';   % Simple linearly space income grid 


% Create combined grid on s=(a,z)
%   - First element "moves faster" than second element, etc 
s    = [repmat(agrid,Nz,1), repelem(zgrid,Na,1)];

% Also find the mesh-grid for use in interpolation
s_a = reshape(s(:,1), Na, Nz);
s_z = reshape(s(:,2), Na, Nz);

%% 0. Pre-allocate value function and choice function vectors

V_j      = zeros(Na*Nz, J);
aprime_j = zeros(Na*Nz, J);
cons_j   = zeros(Na*Nz, J);

%-----------------------------
%% 1. Solve final period of life first: j = J
%-----------------------------

jj = J;  % Set final age 

% Get complete grids for assets and income 
a = s(:,1);
z = s(:,2);

aprime_j(:,jj) = 0;
cons_j(:,jj)   = y_j(jj).*z + a.*(1+r);
V_j(:,jj)      = log( cons_j(:,jj) );

% Note: Will often want to make sure that V_j is never -inf or nan. 
scale     = -1e+10;
V_j(:,jj) = (cons_j(:,jj)>0).*V_j(:,jj) + (cons_j(:,jj)<=0).*scale;


%-----------------------------
%% 2. Solve for all previous ages: j = J-1, ..., 1
%-----------------------------
%   1. Use maximization algorithm to find optimal savings choice 
%       - We use "golden section search" (https://en.wikipedia.org/wiki/Golden-section_search)
%       - GSS is useful because it can be *vectorized*, which massively
%       speeds up the process 
%       - You could use other algorithms, however: Matlab's "fmincon", Newtown methods, hill-climbing methods,
%       Nelder-Meade, etc. Each have pros and cons, but can be difficult to
%       vectorize 
%   2. Now, since we only have values for V_j(jj+1) on the grid, we need to
%   "interpolate" the values of V_j(jj+1) between grid points (https://en.wikipedia.org/wiki/Linear_interpolation)

% Auxilliary functions 
util_func = @(cons) log( cons );

% Loop *backwards* through ages 
for jj = J-1:-1:1
    % Get bounds for asset choice
    lb = amin*ones(Na*Nz,1);          % Lower bound
    ub = (y_j(jj).*z + a.*(1+r)); % Upper bound

    % Get interpolation object: interpolate the future value function
    %   - Now have to interpolate over multiple dimensions! 
    V_j1_int = reshape(V_j(:,jj+1)', Na, Nz);
    V_j1_int = griddedInterpolant(s_a, s_z, V_j1_int, 'linear');    

    % Value function given age, consumption, asset choice 
    val_func      = @(jj, cons, aprime) util_func(cons) + beta.*V_j1_int(aprime,z);    

    % Wrap value function for use in optimization routine below
    cons_func = @(astar) y_j(jj).*z + a.*(1+r) - astar;    
    obj       = @(astar) val_func(jj, cons_func(astar), astar);

    % Use golden-section-search to find optimum asset choice
    [aprime_j(:,jj), V_j(:,jj)] = goldenx(obj,lb,ub);    
    
    % Given optimal choice of assets, reconstruct consumption from budget constraint
    cons_j(:,jj)   = cons_func(aprime_j(:,jj));
end


toc

%% Plot value and choice functions 

V_j_tmp      = reshape(V_j, Na, Nz, J);
cons_j_tmp   = reshape(cons_j, Na, Nz, J);
aprime_j_tmp = reshape(aprime_j, Na, Nz, J);


figure;

%--------------------
%Value functions
%--------------------
subplot(3,3,1)
plot(agrid, V_j_tmp(:,1,1), 'linewidth', 2, 'color', 'blue'); hold on;
plot(agrid, V_j_tmp(:,(Nz/2),1), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot(agrid, V_j_tmp(:,Nz,1), 'linewidth', 2, 'color', [0, 0.5, 0], 'LineStyle', ':');
yy = ylim;
ylim([yy(1)-0.1*abs(yy(1)), yy(2)+0.1*abs(yy(2))])
xlim([agrid(1), agrid(end)])
set(gca,'box','off')
title('Value, age=1')

subplot(3,3,2)
plot(agrid, V_j_tmp(:,1,2), 'linewidth', 2, 'color', 'blue'); hold on;
plot(agrid, V_j_tmp(:,(Nz/2),2), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot(agrid, V_j_tmp(:,Nz,2), 'linewidth', 2, 'color', [0, 0.5, 0], 'LineStyle', ':');
yy = ylim;
ylim([yy(1)-0.1*abs(yy(1)), yy(2)+0.1*abs(yy(2))])
xlim([agrid(1), agrid(end)])
set(gca,'box','off')
title('Value, age=2')


subplot(3,3,3)
plot(agrid, V_j_tmp(:,1,3), 'linewidth', 2, 'color', 'blue'); hold on;
plot(agrid, V_j_tmp(:,(Nz/2),3), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot(agrid, V_j_tmp(:,Nz,3), 'linewidth', 2, 'color', [0, 0.5, 0], 'LineStyle', ':');
yy = ylim;
ylim([yy(1)-0.1*abs(yy(1)), yy(2)+0.1*abs(yy(2))])
xlim([agrid(1), agrid(end)])
legend('$\underline{z}$', '$(\underline{z}+\overline{z})/2$', '$\overline{z}$', 'location', 'southeast');
legend boxoff 
set(gca,'box','off')
title('Value, age=3')




%--------------------
%Consumption functions
%--------------------
subplot(3,3,4)
plot(agrid, cons_j_tmp(:,1,1), 'linewidth', 2, 'color', 'blue'); hold on;
plot(agrid, cons_j_tmp(:,(Nz/2),1), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot(agrid, cons_j_tmp(:,Nz,1), 'linewidth', 2, 'color', [0, 0.5, 0], 'LineStyle', ':');
yy = ylim;
ylim([yy(1)-0.1*abs(yy(1)), yy(2)+0.1*abs(yy(2))])
xlim([agrid(1), agrid(end)])
set(gca,'box','off')
title('Consumption, age=1')

subplot(3,3,5)
plot(agrid, cons_j_tmp(:,1,2), 'linewidth', 2, 'color', 'blue'); hold on;
plot(agrid, cons_j_tmp(:,(Nz/2),2), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot(agrid, cons_j_tmp(:,Nz,2), 'linewidth', 2, 'color', [0, 0.5, 0], 'LineStyle', ':');
yy = ylim;
ylim([yy(1)-0.1*abs(yy(1)), yy(2)+0.1*abs(yy(2))])
xlim([agrid(1), agrid(end)])
set(gca,'box','off')
title('Consumption, age=2')

subplot(3,3,6)
plot(agrid, cons_j_tmp(:,1,3), 'linewidth', 2, 'color', 'blue'); hold on;
plot(agrid, cons_j_tmp(:,(Nz/2),3), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot(agrid, cons_j_tmp(:,Nz,3), 'linewidth', 2, 'color', [0, 0.5, 0], 'LineStyle', ':');
yy = ylim;
ylim([yy(1)-0.1*abs(yy(1)), yy(2)+0.1*abs(yy(2))])
xlim([agrid(1), agrid(end)])
set(gca,'box','off')
title('Consumption, age=3')



%--------------------
%Asset functions
%--------------------
subplot(3,3,7)
plot(agrid, aprime_j_tmp(:,1,1), 'linewidth', 2, 'color', 'blue'); hold on;
plot(agrid, aprime_j_tmp(:,(Nz/2),1), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot(agrid, aprime_j_tmp(:,Nz,1), 'linewidth', 2, 'color', [0, 0.5, 0], 'LineStyle', ':');
yy = ylim;
ylim([yy(1)-0.1*abs(yy(1)), yy(2)+0.1*abs(yy(2))])
xlim([agrid(1), agrid(end)])
set(gca,'box','off')
title('Assets, age=1')

subplot(3,3,8)
plot(agrid, aprime_j_tmp(:,1,2), 'linewidth', 2, 'color', 'blue'); hold on;
plot(agrid, aprime_j_tmp(:,(Nz/2),2), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot(agrid, aprime_j_tmp(:,Nz,2), 'linewidth', 2, 'color', [0, 0.5, 0], 'LineStyle', ':');
yy = ylim;
ylim([yy(1)-0.1*abs(yy(1)), yy(2)+0.1*abs(yy(2))])
xlim([agrid(1), agrid(end)])
set(gca,'box','off')
title('Assets, age=2')

subplot(3,3,9)
plot(agrid, aprime_j_tmp(:,1,3), 'linewidth', 2, 'color', 'blue'); hold on;
plot(agrid, aprime_j_tmp(:,(Nz/2),3), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot(agrid, aprime_j_tmp(:,Nz,3), 'linewidth', 2, 'color', [0, 0.5, 0], 'LineStyle', ':');
yy = ylim;
ylim([yy(1)-0.1*abs(yy(1)), yy(2)+0.1*abs(yy(2))])
xlim([agrid(1), agrid(end)])
set(gca,'box','off')
title('Assets, age=3')


f_height_div  = 1.25;
func_latexfig(f_height_div)


filename = ['Figure_', modelname, '_policyfunc.pdf'];
exportgraphics(gcf, [figdir, filename]);



%-----------------------------
%% 3. Simulate households 
%-----------------------------

Npop = 25000; % Need a large number of households 

sim_assets = zeros(Npop, J);
sim_prod    = zeros(Npop, J);
sim_cons   = zeros(Npop, J);

% Simulate productivity
%   - Independent draws each period
mu_z  = 0;    % Mean of prod dist in period 1
sig_z = 0.25;    % std dev of prod dist in period 1
sim_prod(:,1) = lognrnd(mu_z,sig_z,Npop,1);
sim_prod(:,2) = lognrnd(mu_z,sig_z,Npop,1);
sim_prod(:,3) = lognrnd(mu_z,sig_z,Npop,1);

% Simulate panel one age group at a time
for jj = 1:J

    % Use interpolation over asset and consumption functions to find choices
    aprime_j_int = reshape(aprime_j(:,jj)', Na, Nz);
    aprime_j_int = griddedInterpolant(s_a, s_z, aprime_j_int, 'linear');

    cons_j_int = reshape(cons_j(:,jj)', Na, Nz);
    cons_j_int = griddedInterpolant(s_a, s_z, cons_j_int, 'linear');    

    sim_assets(:,jj+1) = aprime_j_int(sim_assets(:,jj),sim_prod(:,jj));
    sim_cons(:,jj)     = cons_j_int(sim_assets(:,jj),sim_prod(:,jj));    
end
    

%-------------------------
%% Plot life-cycle simulation
%-------------------------

figure;

subplot(3,1,1)
plot(ages, y_j.*mean(sim_prod), 'linewidth', 2, 'color', 'blue', 'Marker', 'o'); hold on;
plot(ages, y_j.*(mean(sim_prod)-1*std(sim_prod)), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot(ages, y_j.*(mean(sim_prod)+1*std(sim_prod)), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
yy = ylim;
ylim([yy(1)*0.9, yy(2)*1.1])
xticks(ages)
title('Life-cycle income')
set(gca,'box','off')

subplot(3,1,2)
plot(ages, mean(sim_cons), 'linewidth', 2, 'color', 'blue', 'Marker', 'o'); hold on;
plot(ages, mean(sim_cons)-1*std(sim_cons), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot(ages, mean(sim_cons)+1*std(sim_cons), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
yy = ylim;
ylim([yy(1)*0.9, yy(2)*1.1])
xticks(ages)
set(gca,'box','off')
title('Life-cycle consumption')


subplot(3,1,3)
plot(ages, mean(sim_assets(:,2:J+1)), 'linewidth', 2, 'color', 'blue', 'Marker', 'o'); hold on;
plot(ages, mean(sim_assets(:,2:J+1))-1*std(sim_assets(:,2:J+1)), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot(ages, mean(sim_assets(:,2:J+1))+1*std(sim_assets(:,2:J+1)), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
yy = ylim;
ylim([yy(1)-0.1*abs(yy(1)), yy(2)+0.1*abs(yy(2))])
xticks(ages)
title('Life-cycle assets')
legend('Mean', 'Mean -/+ 1s.d.', 'location', 'southeast');
legend boxoff 
set(gca,'box','off')



f_height_div  = 1.25;
func_latexfig(f_height_div)

filename = ['Figure_', modelname, '_simlifecycle.pdf'];
exportgraphics(gcf, [figdir, filename]);





%-------------------------
%% Plot simulated distributions
%-------------------------

figure;

subplot(3,1,1)
hist(sim_prod(:), 100);
title('Distribution of Income')
set(gca,'box','off')

subplot(3,1,2)
hist(sim_cons(:), 100);
set(gca,'box','off')
title('Distribution of consumption')

subplot(3,1,3)
hist(sim_assets(:), 100);
title('Distribution of assets')
set(gca,'box','off')

f_height_div  = 1.25;
func_latexfig(f_height_div)

filename = ['Figure_', modelname, '_simdist.pdf'];
exportgraphics(gcf, [figdir, filename]);






