%--------------------------------------------------------------------------
%   Very basic life-cycle (OLG) model 
%   - Deterministic life-cycle income (i.e. no uncertainty)
%   - Savings subject to a borrowing limit 
%   - Solution: 
%       - Savings are chosen/forced onto a pre-determined gridspace 
%       - Works well for very simple models 
%       - Does not work so well for more complicated models 
%   - Simulate a panel of households to approximate the distribution of
%   household assets by age. 
%
%--------------------------------------------------------------------------
%% Setup

modelname = 'mod5';
figdir    = './../Figures/';


%% Parameters 
beta  = 0.95;    % Discount factor 
psi   = 1;     % Value of bequests 
r     = 0.065;    % Interest rate 
a_1   = 0;       % Initial assets 


minage = 20; 
ages = 1:60;  % Ages 
J    = length(ages); % Number of ages 

% Mortality risk 
pi_j = 0.025*(ages./max(ages)).^6;

% Life-cycle profile of income 
y_j = 3 + 5*(ages./max(ages)) - 4*(ages./max(ages)).^4;  % Life-cycle incomes
y_j = y_j./max(y_j);

%% Setup asset gridspace 

tic 

Na    = 100; % Number of asset grid points 
amin  = -4;  % Minimum value of assets on the grid 
amax  = 10;  % Maximum value of assets on the grid 

agrid = linspace(amin, amax, Na)';   % Simple linearly space asset grid 


Nz    = 20; % Number of income grid points 
zmin  = 0.5;  % Minimum value of income on the grid 
zmax  = 5;  % Maximum value of income on the grid 
zgrid = linspace(zmin, zmax, Nz)';   % Simple linearly space income grid 


% Create combined grid on s=(a,z)
%   - First element "moves faster" than second element, etc 
s    = [repmat(agrid,Nz,1), repelem(zgrid,Na,1)];

% Also find the mesh-grid for use in interpolation
s_a = reshape(s(:,1), Na, Nz);
s_z = reshape(s(:,2), Na, Nz);

%% 0. Pre-allocate value function and choice function vectors

V_j      = zeros(Na*Nz, J);
aprime_j = zeros(Na*Nz, J);
cons_j   = zeros(Na*Nz, J);

%-----------------------------
%% 1. Solve final period of life first: j = J
%   - NOTE: This is the final period, *if* still alive
%   - Otherwise bequests will occur like usual 
%-----------------------------

% % % % jj = J;  % Set final age 
% % % % 
% % % % % Get complete grids for assets and income 
% % % % a = s(:,1);
% % % % z = s(:,2);
% % % % 
% % % % aprime_j(:,jj) = 0;
% % % % cons_j(:,jj)   = y_j(jj).*z + a.*(1+r);
% % % % V_j(:,jj)      = log( cons_j(:,jj) );
% % % % 
% % % % % Note: Will often want to make sure that V_j is never -inf or nan. 
% % % % scale     = -1e+10;
% % % % V_j(:,jj) = (cons_j(:,jj)>0).*V_j(:,jj) + (cons_j(:,jj)<=0).*scale;


jj = J+1;  % Set final age 

% Get complete grids for assets and income 
a = s(:,1);
V_j(:,jj)      = psi*log( a );
% Note: Will often want to make sure that V_j is never -inf or nan. 
scale     = -1e+10;
V_j(:,jj) = (a>0).*V_j(:,jj) + (a<=0).*scale;


%-----------------------------
%% 2. Solve for all previous ages: j = J-1, ..., 1
%-----------------------------
%   1. Use maximization algorithm to find optimal savings choice 
%       - We use "golden section search" (https://en.wikipedia.org/wiki/Golden-section_search)
%       - GSS is useful because it can be *vectorized*, which massively
%       speeds up the process 
%       - You could use other algorithms, however: Matlab's "fmincon", Newtown methods, hill-climbing methods,
%       Nelder-Meade, etc. Each have pros and cons, but can be difficult to
%       vectorize 
%   2. Now, since we only have values for V_j(jj+1) on the grid, we need to
%   "interpolate" the values of V_j(jj+1) between grid points (https://en.wikipedia.org/wiki/Linear_interpolation)

% Auxilliary functions 
util_func = @(cons) log( cons );
bequest_func = @(aprime) psi*log( aprime );

% Loop *backwards* through ages 
for jj = J:-1:1
    % Get bounds for asset choice
    lb = amin*ones(Na*Nz,1);          % Lower bound
    ub = (y_j(jj).*z + a.*(1+r)); % Upper bound

    % Get interpolation object: interpolate the future value function
    %   - Now have to interpolate over multiple dimensions! 
    V_j1_int = reshape(V_j(:,jj+1)', Na, Nz);
    V_j1_int = griddedInterpolant(s_a, s_z, V_j1_int, 'linear');    

    % Value function given age, consumption, asset choice 
    val_func      = @(jj, cons, aprime) util_func(cons) ...
                    + beta.*(1-pi_j(jj))*V_j1_int(aprime,z) ...
                    + beta.*pi_j(jj)*bequest_func(aprime);    

    % Wrap value function for use in optimization routine below
    cons_func = @(astar) y_j(jj).*z + a.*(1+r) - astar;    
    obj       = @(astar) val_func(jj, cons_func(astar), astar);

    % Use golden-section-search to find optimum asset choice
    [aprime_j(:,jj), V_j(:,jj)] = goldenx(obj,lb,ub);    
    
    % Given optimal choice of assets, reconstruct consumption from budget constraint
    cons_j(:,jj)   = cons_func(aprime_j(:,jj));
end


toc



%-----------------------------
%% 3. Simulate households 
%-----------------------------

Npop = 25000; % Need a large number of households 

sim_assets = zeros(Npop, J);
sim_prod    = zeros(Npop, J);
sim_cons   = zeros(Npop, J);

% Simulate productivity
%   - Independent draws each period
mu_z  = 1;    % Mean of prod dist in period 1
sig_z = 0.25;    % std dev of prod dist in period 1
for jj = 1:J
    sim_prod(:,jj) = lognrnd(mu_z,sig_z,Npop,1);
end

% Simulate panel one age group at a time
for jj = 1:J

    % Use interpolation over asset and consumption functions to find choices
    aprime_j_int = reshape(aprime_j(:,jj)', Na, Nz);
    aprime_j_int = griddedInterpolant(s_a, s_z, aprime_j_int, 'linear');

    cons_j_int = reshape(cons_j(:,jj)', Na, Nz);
    cons_j_int = griddedInterpolant(s_a, s_z, cons_j_int, 'linear');    

    sim_assets(:,jj+1) = aprime_j_int(sim_assets(:,jj),sim_prod(:,jj));
    sim_cons(:,jj)     = cons_j_int(sim_assets(:,jj),sim_prod(:,jj));    
end
    

%-------------------------
%% Plot life-cycle simulation
%-------------------------

figure;

subplot(3,1,1)
plot(minage+ages, y_j.*mean(sim_prod), 'linewidth', 2, 'color', 'blue', 'Marker', 'o'); hold on;
plot(minage+ages, y_j.*(mean(sim_prod)-2*std(sim_prod)), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot(minage+ages, y_j.*(mean(sim_prod)+2*std(sim_prod)), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
yy = ylim;
ylim([yy(1)*0.9, yy(2)*1.1])
title('Life-cycle income')
set(gca,'box','off')

subplot(3,1,2)
plot(minage+ages, mean(sim_cons), 'linewidth', 2, 'color', 'blue', 'Marker', 'o'); hold on;
plot(minage+ages, mean(sim_cons)-2*std(sim_cons), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot(minage+ages, mean(sim_cons)+2*std(sim_cons), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
yy = ylim;
ylim([yy(1)*0.9, yy(2)*1.1])
set(gca,'box','off')
title('Life-cycle consumption')


subplot(3,1,3)
plot(minage+ages, mean(sim_assets(:,2:J+1)), 'linewidth', 2, 'color', 'blue', 'Marker', 'o'); hold on;
plot(minage+ages, mean(sim_assets(:,2:J+1))-2*std(sim_assets(:,2:J+1)), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot(minage+ages, mean(sim_assets(:,2:J+1))+2*std(sim_assets(:,2:J+1)), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot([minage, minage+ages(end)], [0,0], 'color', 'black', 'LineWidth', 1);
yy = ylim;
ylim([yy(1)-0.1*abs(yy(1)), yy(2)+0.1*abs(yy(2))])
title('Life-cycle assets')
legend('Mean', 'Mean -/+ 2s.d.', 'location', 'southeast');
legend boxoff 
set(gca,'box','off')



f_height_div  = 1.25;
func_latexfig(f_height_div)


filename = ['Figure_', modelname, '_simlifecycle.pdf'];
exportgraphics(gcf, [figdir, filename]);



%-------------------------
%% Plot simulated distributions
%-------------------------

figure;

subplot(3,1,1)
hist(sim_prod(:), 100);
title('Distribution of Income')
set(gca,'box','off')

subplot(3,1,2)
hist(sim_cons(:), 100);
set(gca,'box','off')
title('Distribution of consumption')

subplot(3,1,3)
hist(sim_assets(:), 100);
title('Distribution of assets')
set(gca,'box','off')

f_height_div  = 1.25;
func_latexfig(f_height_div)

filename = ['Figure_', modelname, '_simdist.pdf'];
exportgraphics(gcf, [figdir, filename]);





