%--------------------------------------------------------------------------
%   Very basic life-cycle (OLG) model 
%   - Deterministic life-cycle income (i.e. no uncertainty)
%   - Savings subject to a borrowing limit 
%   - Solution: 
%       - Savings are chosen/forced onto a pre-determined gridspace 
%       - Works well for very simple models 
%       - Does not work so well for more complicated models 
%   - Simulate a panel of households to approximate the distribution of
%   household assets by age. 
%
%--------------------------------------------------------------------------
%% Setup

modelname = 'mod7';
figdir    = './../Figures/';

%% Parameters 
beta  = 0.95;    % Discount factor 
psi   = 1;     % Value of bequests 
r     = 0.065;    % Interest rate 
a_1   = 0;       % Initial assets 
omega_jret = 0.5; % Retirement income


minage = 20; 
ages = 1:60;  % Ages 
J    = length(ages); % Number of ages 
Jret = 45; % Retirement age (i.e. 65)

% Mortality risk 
pi_j = 0.025*(ages./max(ages)).^6;

% Life-cycle profile of income 
y_j = 3 + 5*(ages./max(ages)) - 4*(ages./max(ages)).^4;  % Life-cycle incomes
y_j(Jret+1:end) = y_j(Jret);    % Set retirement age income to final age income 
y_j = y_j./max(y_j);

%% Setup asset gridspace 

tic 

Na    = 100; % Number of asset grid points 
amin  = 0;  % Minimum value of assets on the grid 
amax  = 10;  % Maximum value of assets on the grid 

agrid = linspace(amin, amax, Na)';   % Simple linearly space asset grid 


zgrid = [0.5, 0.85, 1, 1.15, 1.5]';   % Simple linearly space income grid 
Nz    = length(zgrid); % Number of income grid points 

% Markov chain probability transition matrix
Gamma_z  = [0.80, 0.10, 0.075, 0.025, 0; ...
    0.025, 0.90, 0.05, 0.025, 0; ...
    0, 0.025, 0.95, 0.025, 0; ...
    0, 0.025, 0.05, 0.90, 0.025; ...
    0, 0.025, 0.075, 0.10, 0.80];
     
% Create expectations matrix 
% - This creates the expectations over possible transitions from (a,z)-->(a',z')
% - NOTE: There are other ways to do this, e.g. looping over the individual
% states and conditional probabilities of transition. But this is MUCH faster.
E    = kron(Gamma_z, speye(Na));

% Create expectations matrix for retirement 
%   - No more income shocks
Eret = kron(speye(Nz), speye(Na));


% Create combined grid on s=(a,z)
%   - First element "moves faster" than second element, etc 
s    = [repmat(agrid,Nz,1), repelem(zgrid,Na,1)];

% Also find the mesh-grid for use in interpolation
s_a = reshape(s(:,1), Na, Nz);
s_z = reshape(s(:,2), Na, Nz);

%% 0. Pre-allocate value function and choice function vectors

V_j      = zeros(Na*Nz, J);
aprime_j = zeros(Na*Nz, J);
cons_j   = zeros(Na*Nz, J);

%-----------------------------
%% 1. Solve final period of life first: j = J
%   - NOTE: This is the final period, *if* still alive
%   - Otherwise bequests will occur like usual 
%-----------------------------

% % % % jj = J;  % Set final age 
% % % % 
% % % % % Get complete grids for assets and income 
% % % % a = s(:,1);
% % % % z = s(:,2);
% % % % 
% % % % aprime_j(:,jj) = 0;
% % % % cons_j(:,jj)   = y_j(jj).*z + a.*(1+r);
% % % % V_j(:,jj)      = log( cons_j(:,jj) );
% % % % 
% % % % % Note: Will often want to make sure that V_j is never -inf or nan. 
% % % % scale     = -1e+10;
% % % % V_j(:,jj) = (cons_j(:,jj)>0).*V_j(:,jj) + (cons_j(:,jj)<=0).*scale;


jj = J+1;  % Set final age 

% Get complete grids for assets and income 
a = s(:,1);
z = s(:,2); 
V_j(:,jj)      = psi*log( a );
% Note: Will often want to make sure that V_j is never -inf or nan. 
scale     = -1e+10;
V_j(:,jj) = (a>0).*V_j(:,jj) + (a<=0).*scale;


%-----------------------------
%% 2. Solve for all previous ages: j = J-1, ..., 1
%-----------------------------
%   1. Use maximization algorithm to find optimal savings choice 
%       - We use "golden section search" (https://en.wikipedia.org/wiki/Golden-section_search)
%       - GSS is useful because it can be *vectorized*, which massively
%       speeds up the process 
%       - You could use other algorithms, however: Matlab's "fmincon", Newtown methods, hill-climbing methods,
%       Nelder-Meade, etc. Each have pros and cons, but can be difficult to
%       vectorize 
%   2. Now, since we only have values for V_j(jj+1) on the grid, we need to
%   "interpolate" the values of V_j(jj+1) between grid points (https://en.wikipedia.org/wiki/Linear_interpolation)

% Auxilliary functions 
util_func = @(cons) log( cons );
bequest_func = @(aprime) psi*log( aprime );

% Loop *backwards* through ages 
for jj = J:-1:1

    %--------------------
    % RETIRED
    %--------------------
    if jj>Jret
        % Get bounds for asset choice
        lb = amin*ones(Na*Nz,1);          % Lower bound
        ub = omega_jret + a.*(1+r); % Upper bound
    
        % Get interpolation object: interpolate the future value function
        %   - Now have to interpolate over multiple dimensions!
        EV_j1     = Eret*V_j(:,jj+1); 
        EV_j1_int = reshape(EV_j1', Na, Nz);
        EV_j1_int = griddedInterpolant(s_a, s_z, EV_j1_int, 'linear');    
    
        % Value function given age, consumption, asset choice 
        val_func      = @(jj, cons, aprime) util_func(cons) ...
                        + beta.*(1-pi_j(jj))*EV_j1_int(aprime,z) ...
                        + beta.*pi_j(jj)*bequest_func(aprime);    
    
        % Wrap value function for use in optimization routine below
        cons_func = @(astar) omega_jret + a.*(1+r) - astar;            
        obj       = @(astar) val_func(jj, cons_func(astar), astar);
    
        % Use golden-section-search to find optimum asset choice
        [aprime_j(:,jj), V_j(:,jj)] = goldenx(obj,lb,ub);    
        
        % Given optimal choice of assets, reconstruct consumption from budget constraint
        cons_j(:,jj)   = cons_func(aprime_j(:,jj));

    %--------------------
    % WORKING LIFE
    %--------------------        
    elseif jj<=Jret
        % Get bounds for asset choice
        lb = amin*ones(Na*Nz,1);          % Lower bound
        ub = (y_j(jj).*z + a.*(1+r)); % Upper bound
    
        % Get interpolation object: interpolate the future value function
        %   - Now have to interpolate over multiple dimensions!
        EV_j1     = E*V_j(:,jj+1); 
        EV_j1_int = reshape(EV_j1', Na, Nz);
        EV_j1_int = griddedInterpolant(s_a, s_z, EV_j1_int, 'linear');    
    
        % Value function given age, consumption, asset choice 
        val_func      = @(jj, cons, aprime) util_func(cons) ...
                        + beta.*(1-pi_j(jj))*EV_j1_int(aprime,z) ...
                        + beta.*pi_j(jj)*bequest_func(aprime);    
    
        % Wrap value function for use in optimization routine below
        cons_func = @(astar) y_j(jj).*z + a.*(1+r) - astar;    
        obj       = @(astar) val_func(jj, cons_func(astar), astar);
    
        % Use golden-section-search to find optimum asset choice
        [aprime_j(:,jj), V_j(:,jj)] = goldenx(obj,lb,ub);    
        
        % Given optimal choice of assets, reconstruct consumption from budget constraint
        cons_j(:,jj)   = cons_func(aprime_j(:,jj));

    end

end


toc



%-----------------------------
%% 3. Simulate households 
%-----------------------------

Npop = 25000; % Need a large number of households 

sim_assets = zeros(Npop, J);
sim_prod    = zeros(Npop, J);
sim_income  = zeros(Npop, J);
sim_cons   = zeros(Npop, J);

% Simulate productivity

% Get stationary distribution of productivity shocks
Gamma_z_SS = Gamma_z^10000;
Gamma_z_SS = Gamma_z_SS(1,:);

% Draw from stationary distribution for first period
md = makedist('multinomial', Gamma_z_SS/sum(Gamma_z_SS));
z_0 = random(md, Npop, 1);

% Create discrete time markov chain object and simulate person-by-person
mc = dtmc(Gamma_z); 
for ii = 1:Npop
    z0tmp             = zeros(1, Nz); 
    z0tmp(z_0(ii))    = 1;
    sim_prod(ii, :) = zgrid(simulate(mc, J-1, 'X0', z0tmp))';
end
% After retirement, income is constant 
sim_prod(:,Jret+1:end) = repmat(sim_prod(:,Jret), 1, J-Jret); 
% Get income, accounting for retirement 
sim_income(:,1:Jret)     = y_j(1:Jret).*sim_prod(:,1:Jret);
sim_income(:,Jret+1:end) = omega_jret;



% Simulate panel one age group at a time
for jj = 1:J

    % Use interpolation over asset and consumption functions to find choices
    aprime_j_int = reshape(aprime_j(:,jj)', Na, Nz);
    aprime_j_int = griddedInterpolant(s_a, s_z, aprime_j_int, 'linear');

    cons_j_int = reshape(cons_j(:,jj)', Na, Nz);
    cons_j_int = griddedInterpolant(s_a, s_z, cons_j_int, 'linear');    

    sim_assets(:,jj+1) = aprime_j_int(sim_assets(:,jj),sim_prod(:,jj));
    sim_cons(:,jj)     = cons_j_int(sim_assets(:,jj),sim_prod(:,jj));    
end
    

%-------------------------
%% Plot life-cycle simulation
%-------------------------

figure;

subplot(3,1,1)
plot(minage+ages, mean(sim_income), 'linewidth', 2, 'color', 'blue', 'Marker', 'o'); hold on;
plot(minage+ages, (mean(sim_income)-1*std(sim_income)), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot(minage+ages, (mean(sim_income)+1*std(sim_income)), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
yy = ylim;
ylim([yy(1)*0.9, yy(2)*1.1])
title('Life-cycle income')
set(gca,'box','off')

subplot(3,1,2)
plot(minage+ages, mean(sim_cons), 'linewidth', 2, 'color', 'blue', 'Marker', 'o'); hold on;
plot(minage+ages, mean(sim_cons)-1*std(sim_cons), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot(minage+ages, mean(sim_cons)+1*std(sim_cons), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
yy = ylim;
ylim([yy(1)*0.9, yy(2)*1.1])
set(gca,'box','off')
title('Life-cycle consumption')


subplot(3,1,3)
plot(minage+ages, mean(sim_assets(:,2:J+1)), 'linewidth', 2, 'color', 'blue', 'Marker', 'o'); hold on;
plot(minage+ages, mean(sim_assets(:,2:J+1))-1*std(sim_assets(:,2:J+1)), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
plot(minage+ages, mean(sim_assets(:,2:J+1))+1*std(sim_assets(:,2:J+1)), 'linewidth', 2, 'color', 'red', 'LineStyle', '--');
yy = ylim;
ylim([yy(1)-0.1*abs(yy(1)), yy(2)+0.1*abs(yy(2))])
title('Life-cycle assets')
legend('Mean', 'Mean -/+ 1s.d.', 'location', 'southeast');
legend boxoff 
set(gca,'box','off')



f_height_div  = 1.25;
func_latexfig(f_height_div)


filename = ['Figure_', modelname, '_simlifecycle.pdf'];
exportgraphics(gcf, [figdir, filename]);




%-------------------------
%% Plot simulated distributions
%-------------------------

figure;

subplot(3,1,1)
hist(sim_income(:));
title('Distribution of income')
set(gca,'box','off')

subplot(3,1,2)
hist(sim_cons(:), 100);
% xticks(ages)
set(gca,'box','off')
title('Distribution of consumption')


subplot(3,1,3)
hist(sim_assets(:), 100);
title('Distribution of assets')
set(gca,'box','off')

f_height_div  = 1.25;
func_latexfig(f_height_div)


filename = ['Figure_', modelname, '_simdist.pdf'];
exportgraphics(gcf, [figdir, filename]);









